#include "lubeapp.h"

uint32_t lubePulseTimestamp = 0;

void RunLubeApp(volatile uint32_t *wheelPulseCounter)
{
    static uint32_t buttonTimestamp = 0;
    static boolean buttonActionDone = false;

    if (digitalRead(GPIO_BUTTON) == LOW)
    {
        if (buttonTimestamp == 0)
            buttonTimestamp = millis() + BUTTON_HOLD_DELAY;

        if (buttonTimestamp < millis() && buttonActionDone == false)
        {
            switch (globals.systemStatus)
            {
            case sysStat_Normal:
                globals.systemStatus = sysStat_Rain;
                globals.resumeStatus = sysStat_Rain;
                break;

            case sysStat_Rain:
                globals.systemStatus = sysStat_Normal;
                globals.resumeStatus = sysStat_Normal;
                break;
            }
            buttonActionDone = true;
        }
    }
    else
    {
        buttonTimestamp = 0;
        buttonActionDone = false;
    }

    // Calculate traveled Distance in mm
    TravelDistance_highRes += (*wheelPulseCounter * (LubeConfig.DistancePerRevolution_mm / LubeConfig.PulsePerRevolution));
    *wheelPulseCounter = 0;

    // check if we have to Lube!
    switch (globals.systemStatus)
    {
    case sysStat_Startup:
        globals.systemStatus = sysStat_Normal;
        globals.resumeStatus = sysStat_Normal;
        break;

    case sysStat_Normal:
        if (TravelDistance_highRes / 1000 > LubeConfig.DistancePerLube_Default)
        {
            LubePulse();
            TravelDistance_highRes = 0;
        }
        break;

    case sysStat_Rain:
        if (TravelDistance_highRes / 1000 > LubeConfig.DistancePerLube_Rain)
        {
            LubePulse();
            TravelDistance_highRes = 0;
        }
        break;
    case sysStat_Purge:
        if (globals.purgePulses > 0)
        {
            if (lubePulseTimestamp + LUBE_PULSE_PAUSE_MS < millis())
            {
                LubePulse();
                globals.purgePulses--;
            }
        }
        else
        {
            globals.systemStatus = globals.resumeStatus;
        }
        break;
    case sysStat_Error:
    default:
        break;
    }

    // maintain Pin-State of Lube-Pump
    if (lubePulseTimestamp > millis())
        digitalWrite(GPIO_PUMP, HIGH);
    else
        digitalWrite(GPIO_PUMP, LOW);
}

void LubePulse()
{
    lubePulseTimestamp = millis() + LUBE_PULSE_LENGHT_MS;
    LubeConfig.tankRemain_µl = LubeConfig.tankRemain_µl - LubeConfig.amountPerDose_µl;
}